#define CAR_RED_PIN 7
#define CAR_YELLOW_PIN 6
#define CAR_GREEN_PIN 5
#define PED_RED_PIN 4
#define PED_GREEN_PIN 3
#define BUTTON_PIN 2

#define GREEN_YELLOW 1
#define YELLOW_RED 2
#define PED_RED_GREEN 3
#define PED_GREEN_RED 4
#define RED_YELLOW 5
#define YELLOW_GREEN 6
#define PED_GREEN_LOW 7
#define PED_GREEN_HIGH 8

#define NIGHT_MODE_START 9
#define NIGHT_MODE_UP 10
#define NIGHT_MODE_DOWN 11
#define NIGHT_MODE_END 12
#define NIGHT_LIGHT_LEVEL 5

#define BUTTON_PRESSED 1
#define BUTTON_DELAY 40000

volatile int time;
int nightMode;
unsigned long lastInterruptTime = 0;

ISR(INT0_vect) {
  unsigned long interruptTime = millis();

  if (lastInterruptTime == 0 || interruptTime - lastInterruptTime > BUTTON_DELAY) {
    if (time > 74 || time < 22) {
      time = 22;
      lastInterruptTime = interruptTime;
    }
  }
}

ISR(TIMER1_COMPA_vect) {
  time++;
}

void configureInterrupt0() {
  EICRA |= (1 << ISC01);
  EIMSK |= (1 << INT0);
}

void configureADC() {
  ADMUX = 0;
  ADMUX |= (1 << REFS0);

  ADCSRA = 0;
  ADCSRA |= (7 << ADPS0);
  ADCSRA |= (1 << ADEN);
}

void configureTimer1() {
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 31249;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS12);

  TIMSK1 |= (1 << OCIE1A);
}

void setup() {
  Serial.begin(9600);

  pinMode(CAR_RED_PIN, OUTPUT);
  pinMode(CAR_YELLOW_PIN, OUTPUT);
  pinMode(CAR_GREEN_PIN, OUTPUT);
  pinMode(PED_RED_PIN, OUTPUT);
  pinMode(PED_GREEN_PIN, OUTPUT);
  pinMode(BUTTON_PIN, INPUT_PULLUP);

  digitalWrite(CAR_GREEN_PIN, HIGH);
  digitalWrite(PED_RED_PIN, HIGH);

  cli();
  configureInterrupt0();
  configureADC();
  configureTimer1();
  sei();

  time = 0;
  nightMode = 0;
}

void greenYellow() {
  digitalWrite(CAR_GREEN_PIN, LOW);
  digitalWrite(CAR_YELLOW_PIN, HIGH);
}

void yellowRed() {
  digitalWrite(CAR_YELLOW_PIN, LOW);
  digitalWrite(CAR_RED_PIN, HIGH);
}

void pedRedGreen() {
  digitalWrite(PED_RED_PIN, LOW);
  digitalWrite(PED_GREEN_PIN, HIGH);
}

void pedGreenRed() {
  digitalWrite(PED_GREEN_PIN, LOW);
  digitalWrite(PED_RED_PIN, HIGH);
}

void redYellow() {
  digitalWrite(CAR_RED_PIN, LOW);
  digitalWrite(CAR_YELLOW_PIN, HIGH);
}

void yellowGreen() {
  digitalWrite(CAR_YELLOW_PIN, LOW);
  digitalWrite(CAR_GREEN_PIN, HIGH);
}

uint16_t analogAdcRead() {
  ADCSRA |= (1 << ADSC);
  while (!(ADCSRA & (1 << ADIF)));
  return ADC;
}

void loop() {
  uint16_t lightLevel = analogAdcRead();

  if (lightLevel < NIGHT_LIGHT_LEVEL) {
    nightMode = 1;
    digitalWrite(CAR_RED_PIN, LOW);
    digitalWrite(CAR_GREEN_PIN, LOW);
    digitalWrite(PED_RED_PIN, LOW);
    digitalWrite(PED_GREEN_PIN, LOW);
    Serial.write(NIGHT_MODE_START);

    if (time % 2 == 0) {
      digitalWrite(CAR_YELLOW_PIN, LOW);
      Serial.write(NIGHT_MODE_DOWN);
    } else {
      digitalWrite(CAR_YELLOW_PIN, HIGH);
      Serial.write(NIGHT_MODE_UP);
    }
  } else {
    if (nightMode == 1) {
      nightMode = 0;

      digitalWrite(CAR_YELLOW_PIN, LOW);
      digitalWrite(CAR_GREEN_PIN, HIGH);
      digitalWrite(PED_RED_PIN, HIGH);
      Serial.write(NIGHT_MODE_END);

      time = 0;
    }

    if (Serial.available() > 0) {
      int incomingState = Serial.read();

      if (incomingState == BUTTON_PRESSED) {
        unsigned long interruptTime = millis();

        if (lastInterruptTime == 0 || interruptTime - lastInterruptTime > BUTTON_DELAY) {
          if (time > 34 && time < 62) {
            time = 62;
            lastInterruptTime = interruptTime;
          }
        }
      }
    }

    switch (time) {
      case 28:
        greenYellow();
        Serial.write(PED_GREEN_LOW);
        break;
      case 22:
      case 24:
      case 26:
      case 30:
        Serial.write(PED_GREEN_LOW);
        break;
      case 23:
      case 25:
      case 27:
      case 29:
      case 31:
        Serial.write(PED_GREEN_HIGH);
        break;
      case 32:
        Serial.write(4);
        break;
      case 34:
        yellowRed();
        break;
      case 38:
        Serial.write(RED_YELLOW);
        break;
      case 40:
        pedRedGreen();
        Serial.write(YELLOW_GREEN);
        break;
      case 68:
        digitalWrite(PED_GREEN_PIN, LOW);
        Serial.write(GREEN_YELLOW);
        break;
      case 62:
      case 64:
      case 66:
      case 70:
        digitalWrite(PED_GREEN_PIN, LOW);
        break;
      case 63:
      case 65:
      case 67:
      case 69:
      case 71:
        digitalWrite(PED_GREEN_PIN, HIGH);
        break;
      case 72:
        pedGreenRed();
        break;
      case 74:
        Serial.write(YELLOW_RED);
        break;
      case 78:
        redYellow();
        break;
      case 80:
        yellowGreen();
        Serial.write(PED_RED_GREEN);
        time = 0;
        break;
    }
  }
}
