#define CAR_RED_PIN 7
#define CAR_YELLOW_PIN 6
#define CAR_GREEN_PIN 5
#define PED_RED_PIN 4
#define PED_GREEN_PIN 3
#define BUTTON_PIN 2

#define GREEN_YELLOW 1
#define YELLOW_RED 2
#define PED_RED_GREEN 3
#define PED_GREEN_RED 4
#define RED_YELLOW 5
#define YELLOW_GREEN 6
#define PED_GREEN_LOW 7
#define PED_GREEN_HIGH 8

#define NIGHT_MODE_START 9
#define NIGHT_MODE_UP 10
#define NIGHT_MODE_DOWN 11
#define NIGHT_MODE_END 12

#define BUTTON_PRESSED 1
#define DEBOUNCE_TIME 1000

ISR(INT0_vect) {
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();

  if (last_interrupt_time == 0 || interrupt_time - last_interrupt_time > DEBOUNCE_TIME) {
    Serial.write(BUTTON_PRESSED);
  }
  last_interrupt_time = interrupt_time;
}

void configure_interrupt0() {
  EICRA |= (1 << ISC01);
  EIMSK |= (1 << INT0);
}

void setup() {
  Serial.begin(9600);

  pinMode(CAR_RED_PIN, OUTPUT);
  pinMode(CAR_YELLOW_PIN, OUTPUT);
  pinMode(CAR_GREEN_PIN, OUTPUT);
  pinMode(PED_RED_PIN, OUTPUT);
  pinMode(PED_GREEN_PIN, OUTPUT);
  pinMode(BUTTON_PIN, INPUT_PULLUP);

  digitalWrite(CAR_RED_PIN, HIGH);
  digitalWrite(PED_GREEN_PIN, HIGH);

  cli();
  configure_interrupt0();
  sei();
}

void green_yellow() {
  digitalWrite(CAR_GREEN_PIN, LOW);
  digitalWrite(CAR_YELLOW_PIN, HIGH);
}

void yellow_red() {
  digitalWrite(CAR_YELLOW_PIN, LOW);
  digitalWrite(CAR_RED_PIN, HIGH);
}

void ped_red_green() {
  digitalWrite(PED_RED_PIN, LOW);
  digitalWrite(PED_GREEN_PIN, HIGH);
}

void ped_green_red() {
  digitalWrite(PED_GREEN_PIN, LOW);
  digitalWrite(PED_RED_PIN, HIGH);
}

void red_yellow() {
  digitalWrite(CAR_RED_PIN, LOW);
  digitalWrite(CAR_YELLOW_PIN, HIGH);
}

void yellow_green() {
  digitalWrite(CAR_YELLOW_PIN, LOW);
  digitalWrite(CAR_GREEN_PIN, HIGH);
}

void loop() {
  if (Serial.available() > 0) {
    int incoming_state = Serial.read();

    switch (incoming_state) {
      case GREEN_YELLOW:
        green_yellow();
        break;
      case YELLOW_RED:
        yellow_red();
        break;
      case PED_RED_GREEN:
        ped_red_green();
        break;
      case PED_GREEN_RED:
        ped_green_red();
        break;
      case RED_YELLOW:
        red_yellow();
        break;
      case YELLOW_GREEN:
        yellow_green();
        break;
      case PED_GREEN_LOW:
        digitalWrite(PED_GREEN_PIN, LOW);
        break;
      case PED_GREEN_HIGH:
        digitalWrite(PED_GREEN_PIN, HIGH);
        break;
      case NIGHT_MODE_START:
        digitalWrite(CAR_RED_PIN, LOW);
        digitalWrite(CAR_GREEN_PIN, LOW);
        digitalWrite(PED_RED_PIN, LOW);
        digitalWrite(PED_GREEN_PIN, LOW);
        break;
      case NIGHT_MODE_DOWN:
        digitalWrite(CAR_YELLOW_PIN, LOW);
        break;
      case NIGHT_MODE_UP:
        digitalWrite(CAR_YELLOW_PIN, HIGH);
        break;
      case NIGHT_MODE_END:
        digitalWrite(CAR_YELLOW_PIN, LOW);
        digitalWrite(CAR_RED_PIN, HIGH);
        digitalWrite(PED_GREEN_PIN, HIGH);
        break;
    }
  }
}